﻿using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using System.Reflection;
using System;

namespace AJAXEnableSharePoint
{
    class AJAXEnableReceiver : SPFeatureReceiver
    {
        /// <summary>
        /// The owner that we will assign to the web config modifications
        /// </summary>
        private static string modificationOwner = Assembly.GetExecutingAssembly().FullName;

        /// <summary>
        /// Array of the modifications to make to the web.config
        /// </summary>
        private SPWebConfigModification[] modifications = 
        {
            new SPWebConfigModification() 
            {
                Name = "sectionGroup[@name=\"system.web.extensions\"]",
                Path = "configuration/configSections",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = @"<sectionGroup name=""system.web.extensions"" type=""System.Web.Configuration.SystemWebExtensionsGroup, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"">
      <sectionGroup name=""scripting"" type=""System.Web.Configuration.ScriptingSectionGroup, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"">
        <section name=""scriptResourceHandler"" type=""System.Web.Configuration.ScriptingScriptResourceHandlerSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"" requirePermission=""false"" allowDefinition=""MachineToApplication"" />
        <sectionGroup name=""webServices"" type=""System.Web.Configuration.ScriptingWebServicesSectionGroup, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"">
          <section name=""authenticationService"" type=""System.Web.Configuration.ScriptingAuthenticationServiceSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"" requirePermission=""false"" allowDefinition=""MachineToApplication"" />
          <section name=""jsonSerialization"" type=""System.Web.Configuration.ScriptingJsonSerializationSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"" requirePermission=""false"" allowDefinition=""Everywhere"" />
          <section name=""profileService"" type=""System.Web.Configuration.ScriptingProfileServiceSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"" requirePermission=""false"" allowDefinition=""MachineToApplication"" />
        </sectionGroup>
      </sectionGroup>
    </sectionGroup>
"
            },
            new SPWebConfigModification()
            {
                Name = "controls",
                Path = "configuration/system.web/pages",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<controls></controls>"
            },
            new SPWebConfigModification()
            {
                Name = "add[@assembly=\"System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"]",
                Path = "configuration/system.web/pages/controls",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<add tagPrefix=\"asp\" namespace=\"System.Web.UI\" assembly=\"System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" />"
            },
            new SPWebConfigModification()
            {
                Name = "add[@assembly=\"System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"]",
                Path = "configuration/system.web/compilation/assemblies",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<add assembly=\"System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" />"
            },
            new SPWebConfigModification()
            {
                Name = "add[@path=\"*.asmx\"]",
                Path = "configuration/system.web/httpHandlers",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<add verb=\"*\" path=\"*.asmx\" validate=\"false\" type=\"System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" />"
            },
            new SPWebConfigModification()
            {
                Name = "add[@path=\"*_AppService.axd\"]",
                Path = "configuration/system.web/httpHandlers",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<add verb=\"*\" path=\"*_AppService.axd\" validate=\"false\" type=\"System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" />"
            },
            new SPWebConfigModification()
            {
                Name = "add[@path=\"ScriptResource.axd\"]",
                Path = "configuration/system.web/httpHandlers",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<add verb=\"GET,HEAD\" path=\"ScriptResource.axd\" validate=\"false\" type=\"System.Web.Handlers.ScriptResourceHandler, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" />"
            },
            new SPWebConfigModification()
            {
                Name = "add[@name=\"ScriptModule\"]",
                Path = "configuration/system.web/httpModules",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<add name=\"ScriptModule\" type=\"System.Web.Handlers.ScriptModule, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" />"
            },
            new SPWebConfigModification()
            {
                Name = "SafeControl[@Assembly=\"System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\"]",
                Path = "configuration/SharePoint/SafeControls",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<SafeControl Assembly=\"System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35\" Namespace=\"System.Web.UI\" TypeName=\"*\" Safe=\"True\" />"
            },
            new SPWebConfigModification()
            {
                Name = "system.web.extensions",
                Path = "configuration",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = @"<system.web.extensions>
    <scripting>
      <webServices>
      <!-- Uncomment this line to enable the authentication service. Include requireSSL=""true"" if appropriate. -->
      <!--
        <authenticationService enabled=""true"" requireSSL = ""true|false""/>
      -->
      <!-- Uncomment these lines to enable the profile service. To allow profile properties to be retrieved and modified in ASP.NET AJAX applications, you need to add each property name to the readAccessProperties and writeAccessProperties attributes. -->
      <!--
      <profileService enabled=""true""
                      readAccessProperties=""propertyname1,propertyname2""
                      writeAccessProperties=""propertyname1,propertyname2"" />
      -->
      </webServices>
      <!--
      <scriptResourceHandler enableCompression=""true"" enableCaching=""true"" />
      -->
    </scripting>
  </system.web.extensions>
  "
            },
            new SPWebConfigModification()
            {
                Name = "system.webServer",
                Path = "configuration",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = @"<system.webServer>
    <validation validateIntegratedModeConfiguration=""false""/>
    <modules>
      <add name=""ScriptModule"" preCondition=""integratedMode"" type=""System.Web.Handlers.ScriptModule, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35""/>
    </modules>
    <handlers>
      <remove name=""WebServiceHandlerFactory-Integrated"" />
      <add name=""ScriptHandlerFactory"" verb=""*"" path=""*.asmx"" preCondition=""integratedMode""
           type=""System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35""/>
      <add name=""ScriptHandlerFactoryAppServices"" verb=""*"" path=""*_AppService.axd"" preCondition=""integratedMode"" type=""System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35""/>
      <add name=""ScriptResource"" preCondition=""integratedMode"" verb=""GET,HEAD"" path=""ScriptResource.axd"" type=""System.Web.Handlers.ScriptResourceHandler, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"" />
    </handlers>
  </system.webServer>
"
            },
            new SPWebConfigModification() 
            {
                Name = "add[@assembly=\"AjaxControlToolkit, Version=3.0.20820.16598, Culture=neutral, PublicKeyToken=28f01b0e84b6d53e\"]",
                Path = "configuration/system.web/compilation/assemblies",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<add assembly=\"AjaxControlToolkit, Version=3.0.20820.16598, Culture=neutral, PublicKeyToken=28f01b0e84b6d53e\" />"
            },
            new SPWebConfigModification()
            {
                Name = "add[@assembly=\"AjaxControlToolkit, Version=3.0.20820.16598, Culture=neutral, PublicKeyToken=28f01b0e84b6d53e\"]",
                Path = "configuration/system.web/pages/controls",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<add tagPrefix=\"ajaxToolkit\" namespace=\"AjaxControlToolkit\" assembly=\"AjaxControlToolkit, Version=3.0.20820.16598, Culture=neutral, PublicKeyToken=28f01b0e84b6d53e\" />"
            },
            new SPWebConfigModification()
            {
                Name = "SafeControl[@Namespace=\"AjaxControlToolkit.MaskedEditValidatorCompatibility\"]",
                Path = "configuration/SharePoint/SafeControls",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<SafeControl Assembly=\"AjaxControlToolkit, Version=3.0.20820.16598, Culture=neutral, PublicKeyToken=28f01b0e84b6d53e\" Namespace=\"AjaxControlToolkit.MaskedEditValidatorCompatibility\" TypeName=\"*\" Safe=\"True\" />"
            },
            new SPWebConfigModification()
            {
                Name = "SafeControl[@Namespace=\"AjaxControlToolkit\"]",
                Path = "configuration/SharePoint/SafeControls",
                Owner = modificationOwner,
                Sequence = 0,
                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                Value = "<SafeControl Assembly=\"AjaxControlToolkit, Version=3.0.20820.16598, Culture=neutral, PublicKeyToken=28f01b0e84b6d53e\" Namespace=\"AjaxControlToolkit\" TypeName=\"*\" Safe=\"True\" />"
            }
        };

        /// <summary>
        /// Makes the web config modifications specified in the modifications array.
        /// Fired off when the feature is activated on a web application.
        /// </summary>
        /// <param name="properties"></param>
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            // Get a handle to the web application
            SPWebApplication thisWebApplication = (SPWebApplication)properties.Feature.Parent;

            try
            {
                // Check for existence of System.Web.Extensions
                Assembly.Load(new AssemblyName("System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"));

                // If an exception hasn't been thrown, we found it... let's make appropriate web.config entries
                for (int i = 0; i < modifications.Length; i++)
                {
                    thisWebApplication.WebConfigModifications.Add(modifications[i]);
                }

                // Save the modifications to the web.config
                thisWebApplication.Farm.Services.GetValue<SPWebService>().ApplyWebConfigModifications();
                thisWebApplication.Update();
            }
            catch (Exception ex)
            {
                // Throw exception up to the GUI
                throw ex;
            }
        }

        /// <summary>
        /// Removes the web config modifications that have the owner specified in the modifications array
        /// Fired off when the feature is deactivated on a web application.
        /// </summary>
        /// <param name="properties"></param>
        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            // Get a handle to the web application
            SPWebApplication thisWebApplication = (SPWebApplication)properties.Feature.Parent;

            try
            {
                // Remove all modifications w/ that have this assembly as the owner
                for (int i = 0; i < thisWebApplication.WebConfigModifications.Count; i++)
                {
                    if (thisWebApplication.WebConfigModifications[i].Owner == modificationOwner)
                    {
                        thisWebApplication.WebConfigModifications.RemoveAt(i);
                        i--;
                    }
                }

                // Remove the modifications from the web.config
                thisWebApplication.Farm.Services.GetValue<SPWebService>().ApplyWebConfigModifications();
                thisWebApplication.Update();
            }
            catch (Exception ex)
            {
                // Throw exception up to the GUI
                throw ex;
            }
        }

        public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        {
        }

        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        {
        }
    }
}
